// DlgODBC.cpp : implementation file
//

#include "stdafx.h"
#include "TryODBC.h"
#include "DlgODBC.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CTryODBCDlg dialog

CTryODBCDlg::CTryODBCDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CTryODBCDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CTryODBCDlg)
	m_IntegerField = 0;
	m_FloatField = 0.0f;
	m_CharField = _T("");
	m_TimeField = 0;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
}

void CTryODBCDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTryODBCDlg)
	DDX_Control(pDX, IDC_LIST, m_List);
	DDX_Text(pDX, IDC_INTEGER_FIELD, m_IntegerField);
	DDX_Text(pDX, IDC_FLOAT_FIELD, m_FloatField);
	DDX_Text(pDX, IDC_CHAR_FIELD, m_CharField);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CTryODBCDlg, CDialog)
	//{{AFX_MSG_MAP(CTryODBCDlg)
	ON_BN_CLICKED(IDC_TRY_INSERT, OnTryInsert)
	ON_BN_CLICKED(IDC_TRY_SELECT, OnTrySelect)
	ON_BN_CLICKED(IDC_TRY_SELECT2, OnTrySelect2)
	ON_BN_CLICKED(IDC_TRY_DELETE, OnTryDelete)
	ON_BN_CLICKED(IDC_TRY_UPDATE, OnTryUpdate)
	ON_BN_CLICKED(IDC_CONNECT, OnConnect)
	ON_BN_CLICKED(IDC_DISCONNECT, OnDisconnect)
	ON_BN_CLICKED(IDC_TRY_UPDATE2, OnTryUpdate2)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTryODBCDlg message handlers

BOOL CTryODBCDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

    // Initialize list control
    m_List.InsertColumn(0,"ID",LVCFMT_LEFT,50);
    m_List.InsertColumn(1,"IntegerField",LVCFMT_LEFT,50);
    m_List.InsertColumn(2,"FloatField",LVCFMT_LEFT,100);
    m_List.InsertColumn(3,"CharField",LVCFMT_LEFT,150);
    m_List.InsertColumn(4,"TimeField",LVCFMT_LEFT,200);

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// Add a record to list control
void CTryODBCDlg::AddToList(int ID,int IntegerField,float FloatField,LPCSTR CharField,CTime TimeField)
{
   CString Buffer;
   int Index=m_List.GetItemCount();

   Buffer.Format("%i",ID);
   m_List.InsertItem(LVIF_TEXT,Index,Buffer,0,0,0,0);

   Buffer.Format("%i",IntegerField);
   m_List.SetItemText(Index,1,Buffer);

   Buffer.Format("%f",FloatField);
   m_List.SetItemText(Index,2,Buffer);

   Buffer.Format("%s",CharField);
   m_List.SetItemText(Index,3,Buffer);

   Buffer=TimeField.Format( "%A, %B %d, %Y, %H:%M:%S" );
   m_List.SetItemText(Index,4,Buffer);
}


// Enable/Disable buttons
void CTryODBCDlg::EnableButtons(BOOL On)
{
	GetDlgItem(IDC_CONNECT)->EnableWindow(!On);
    GetDlgItem(IDC_DISCONNECT)->EnableWindow(On);
	GetDlgItem(IDC_TRY_SELECT)->EnableWindow(On);
	GetDlgItem(IDC_TRY_SELECT2)->EnableWindow(On);
	GetDlgItem(IDC_TRY_INSERT)->EnableWindow(On);
	GetDlgItem(IDC_TRY_DELETE)->EnableWindow(On);
	GetDlgItem(IDC_TRY_UPDATE)->EnableWindow(On);
	GetDlgItem(IDC_TRY_UPDATE2)->EnableWindow(On);
}

//=============================================================================
// Beginning ODBC part
//=============================================================================

// Connect to database
void CTryODBCDlg::OnConnect() 
{
	if (m_Connection.Connect("DSN=TRYODBC;UID=USER;"))
		EnableButtons(TRUE);
	else
		AfxMessageBox("Cannot connect to ODBC data source TRYODBC, did you define it in Control Panel's ODBC 32?");
}


// Disconnect from database
void CTryODBCDlg::OnDisconnect() 
{
   if (m_Connection.Disconnect())
	   EnableButtons(FALSE);
}

// Insert a record in the table
void CTryODBCDlg::OnTryInsert() 
{
	// Get data from controls
    if (!UpdateData(TRUE)) return;

    ODBCStatement Stmt(&m_Connection,"OnTryInsert");

	TIMESTAMP_STRUCT TimeStamp; 
	CTime_to_TIMESTAMP_STRUCT(CTime::GetCurrentTime(),TimeStamp); // Now

	// Bind the parameters to the buffers
    Stmt.BindParameter(1,&m_IntegerField);
	Stmt.BindParameter(2,&m_FloatField);
	Stmt.BindParameter(3,m_CharField);
    Stmt.BindParameter(4,&TimeStamp);

    Stmt.ExecDirect("INSERT INTO [TryODBC] ([IntegerField],[FloatField],[CharField],[TimeField]) "
		                "VALUES (?,?,?,?)");
}

// Delete all records from the table
void CTryODBCDlg::OnTryDelete() 
{
    // Empty list control
    m_List.DeleteAllItems(); 

    // Do ODBC stuff, run SQL statement
    ODBCStatement Stmt(&m_Connection);
    Stmt.ExecDirect("DELETE FROM [TryODBC]"); // Deletes all records
}

// Update records increasing by 1 the IntegerField
void CTryODBCDlg::OnTryUpdate() 
{
    ODBCStatement Stmt(&m_Connection);
    Stmt.ExecDirect("UPDATE [TryODBC] SET [IntegerField]=[IntegerField]+1;");
}

// Increment IntegerField if it is less than in dialog box
void CTryODBCDlg::OnTryUpdate2() 
{
	// Get data from controls
    UpdateData(TRUE);
	
	// Do ODBC stuff, running SQL statement
    ODBCStatement Stmt(&m_Connection);
	Stmt.BindParameter(1,&m_IntegerField);
    Stmt.ExecDirect("UPDATE [TryODBC] SET [IntegerField]=[IntegerField]+1 WHERE [IntegerField]<?;");
}


// Select records using BindCol
void CTryODBCDlg::OnTrySelect() 
{
	// Empty list control
    m_List.DeleteAllItems();

    ODBCStatement Stmt(&m_Connection,"OnTrySelect");
	   
	int     ID;
	int     IntegerField;
	char    CharField[256];
	float   FloatField;
	TIMESTAMP_STRUCT TimeStamp;
	CTime   TimeField;

	// Bind the columns of the result set to the buffers
	Stmt.BindCol(1,&ID);
	Stmt.BindCol(2,&IntegerField);
	Stmt.BindCol(3,&FloatField);
	Stmt.BindCol(4,CharField,sizeof(CharField));
    Stmt.BindCol(5,&TimeStamp);

    if (Stmt.ExecDirect("SELECT [ID],[IntegerField],[FloatField],[CharField],[TimeField] "
		                   "FROM [TryODBC]"))
	   while (Stmt.Fetch())
       {
          TIMESTAMP_STRUCT_to_CTime(TimeStamp,TimeField);
	  	  AddToList(ID,IntegerField,FloatField,CharField,TimeField);
	   }
}

// Select records using GetData
void CTryODBCDlg::OnTrySelect2() 
{
	// Empty list control
    m_List.DeleteAllItems();

    ODBCStatement Stmt(&m_Connection,"OnTrySelect2");
	   
    if (Stmt.ExecDirect("SELECT [ID],[IntegerField],[FloatField],[CharField],[TimeField] "
		                   "FROM [TryODBC]"))
	   while (Stmt.Fetch())
       {
		 int     ID;
		 int     IntegerField;
		 float   FloatField;
		 CString CharField;
		 CTime   TimeField;

		 // Get the data from the result set into the buffers
		 Stmt.GetData(1,&ID);
		 Stmt.GetData(2,&IntegerField);
		 Stmt.GetData(3,&FloatField);
		 Stmt.GetData(4,&CharField);
		 Stmt.GetData(5,&TimeField);

	  	 AddToList(ID,IntegerField,FloatField,CharField,TimeField);
	   }
}



// Handle errors and warnings generated by our ODBC classes
BOOL ReportODBCProblem(BOOL isError,LPCSTR function,LPCSTR error,LPCSTR state,LPCSTR retcode,LPCSTR command,LPCSTR extra,long native,long row)
{ 
  ASSERT(function && error && state && retcode && command && extra);
  
  CString All;
  char msg[4096],piece[256]; 
  
  sprintf(msg,"ODBC %s: Function %s returned %s with state %s native error %li %s",
           isError?"Error":"Warning",function,retcode,state,native,extra);
  ASSERT(lstrlen(msg)<sizeof(msg));         

  All=msg;
  All+="\r\n";
  All+=error;
  All+="\r\n";
  if (command && *command) 
     { All+=command; All+="\r\n"; }
  if (row)
     All+=(sprintf(piece,"Row %li\r\n",row),piece);      
  ASSERT(lstrlen(msg)<sizeof(msg));              
  
  if (isError) 
     MessageBox(AfxGetMainWnd()->GetSafeHwnd(),All,"ODBC Error",MB_OK|MB_ICONSTOP); 
  else
     TRACE(All);
        
  return 0;
}

