//////////////////////////////////////////////////////////////////////
// A Simple XML Parser
//
// See XmlReader.h
//////////////////////////////////////////////////////////////////////
//
// Copyright (c) 1999 Sebastien Andrivet. All rights reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// ``Software''), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED ``AS IS'', WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL JAMES CLARK BE LIABLE FOR ANY CLAIM, DAMAGES OR
// OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.

// Except as contained in this notice, the name of Sebastien Andrivet
// shall not be used in advertising or otherwise to promote the sale, 
// use or other dealings in this Software without prior written 
// authorization from Sebastien Andrivet.
//////////////////////////////////////////////////////////////////////

#ifndef ASSERT
#define ASSERT(expr)
#endif

//////////////////////////////////////////////////////////////////////
// Helpers
//////////////////////////////////////////////////////////////////////

inline bool IsSpace(Char c)
// Space, tabulation, line feed or return
{
    return(c == TEXT(' ') || c == TEXT('\t') || c == TEXT('\r') || c == TEXT('\n'));
}

inline bool IsAlpha(Char c)
// [a-zA-Z]
{
    return((c >= TEXT('a') && c <= TEXT('z')) || (c >= TEXT('A') && c <= TEXT('Z')));
}

inline bool IsDigit(Char c)
// [0-9]
{
    return(c >= TEXT('0') && c <= TEXT('9'));
}

inline bool IsHexDigit(Char c)
// [0-9a-fA-F]
{
    return(IsDigit(c) || (c >= TEXT('a') && c <= TEXT('f')) || (c >= TEXT('A') && c <= TEXT('F')));
}

inline int HexDigitValue(Char c)
// [0-9a-fA-F]
{
    return((c >= TEXT('0') && c <= TEXT('9')) ? c - TEXT('0')
        : ((c >= TEXT('a') && c <= TEXT('f')) ? c - TEXT('a') + 10
        : c - TEXT('A') + 10));
        
}

inline bool IsAlphaDigit(Char c)
// [a-zA-Z0-9]
{
    return(IsAlpha(c) || IsDigit(c));
}

inline bool IsAlphaDigitEx(Char c)
// [a-zA-Z0-9_.:] | '-'
{
    return(IsAlphaDigit(c) || c == TEXT('_') || c == TEXT('.') || c == TEXT(':') || c == TEXT('-'));
}

inline Char LowCase(Char c)
{
    return(c >= TEXT('A') && c <= TEXT('Z') ? c - TEXT('A') + TEXT('a') : c);
}

//////////////////////////////////////////////////////////////////////

inline bool IsXmlChar(Char c)
{
    return(c == 0x9 || c == 0xa || c == 0xd  || c >= 0x20);
}

//////////////////////////////////////////////////////////////////////
// Attribute of a tag
//////////////////////////////////////////////////////////////////////

inline const string& Attribute::GetName() const
{
    return(m_strName);
}

inline const string& Attribute::GetValue() const
{
    return(m_strValue);
}

//////////////////////////////////////////////////////////////////////
// Element
//////////////////////////////////////////////////////////////////////

inline const string& Element::GetName() const
{
    return(m_strName);
}

inline const Value& Element::GetValue() const
{
    return(m_value);
}

inline void Element::AddValue(const string& strText)
{
    m_value.Add(strText);
}

inline void Element::AddValue(Char c)
{
    m_value.Add(c);
}

inline const Element& Element::operator()(PCSTR szName, int nIndex) const
{
    return(GetChild(szName, nIndex));
}

//////////////////////////////////////////////////////////////////////
// ElementTag
//////////////////////////////////////////////////////////////////////

inline void ElementTag::Add(Attribute* pAttribute)
{
    m_attributes.push_back(pAttribute);
}

//////////////////////////////////////////////////////////////////////
// Exception
//////////////////////////////////////////////////////////////////////

inline Exception::Exception(int nLine, int nColumn)
:   m_nLine(nLine),
    m_nColumn(nColumn)
{
}

inline int Exception::GetLine() const
{
    return(m_nLine);
}

inline int Exception::GetColumn() const
{
    return(m_nColumn);
}

//////////////////////////////////////////////////////////////////////
// Bookmark
//////////////////////////////////////////////////////////////////////

inline Parser::Bookmark::Bookmark(Parser& reader)
:   m_reader(reader),
    m_szSourceCurrent(reader.m_szSourceCurrent)
{
}

inline void Parser::Bookmark::Restore()
{
    m_reader.m_szSourceCurrent = m_szSourceCurrent;
}

inline void Parser::Bookmark::GetSubString(string& strString, int nNumEndSkip)
{
    ASSERT(m_reader.m_szSourceCurrent + nNumEndSkip >= m_szSourceCurrent);
    strString = string(m_szSourceCurrent, m_reader.m_szSourceCurrent - m_szSourceCurrent - nNumEndSkip);
}

inline void Parser::Bookmark::Reset()
{
    m_szSourceCurrent = m_reader.m_szSourceCurrent;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////


